/***************************************************************************/
/* Routine to simplify parsing of XML documents.  Based more on occurance  */
/* and node/attribute names than on actually forcing the implementor to    */
/* walk the tree.                                                          */
/* This routine should be run as a persistent procedure.                   */
/***************************************************************************/
DEFINE VARIABLE RCS AS CHARACTER
 INIT "$Header: /appl/DOM/src/RCS/DOMParse.p,v 1.1 2007/03/17 14:51:12 sauge Exp sauge $".

DEFINE VARIABLE hXMLDoc   AS HANDLE NO-UNDO.
DEFINE VARIABLE hRootNode AS HANDLE NO-UNDO.
DEFINE VARIABLE hTrvNode  AS HANDLE NO-UNDO.

DEFINE VARIABLE gcErrMsg  AS CHARACTER NO-UNDO.

/***************************************************************************/
/* Error handling so we can see what is going on in here.                  */
/***************************************************************************/

PROCEDURE GetError:

  DEFINE OUTPUT PARAMETER cErrMsg AS CHARACTER NO-UNDO.

  ASSIGN cErrMsg = gcErrMsg.

END.

PROCEDURE SetError:
  DEFINE INPUT PARAMETER cErrNo AS CHARACTER NO-UNDO.

  IF cErrNo = "000" THEN ASSIGN gcErrMsg = "000:No Error".

  IF cErrNo = "100" THEN ASSIGN gcErrMsg = "100:Couldn't Load XML File".

  IF cErrNo = "101" THEN ASSIGN gcErrMsg = "101:Couldn't find root node".

END.

/***************************************************************************/
/* Load the XML document into this instance's work space.                  */
/***************************************************************************/

PROCEDURE LoadXMLDoc:

  DEFINE INPUT PARAMETER cFileName AS CHARACTER NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").

  CREATE X-DOCUMENT hXMLDoc.
  CREATE X-NODEREF hRootNode.

  IF hXMLDoc:LOAD ("FILE", cFileName, FALSE) = FALSE THEN DO:
    RUN SetError IN THIS-PROCEDURE ("100").
    RETURN.
  END.
                                                            

  IF hXMLDoc:GET-DOCUMENT-ELEMENT(hRootNode) = FALSE THEN DO:
    RUN SetError IN THIS-PROCEDURE ("101").
    RETURN.
  END.

END. /* PROCEDURE LoadXMLDoc */        

/***************************************************************************/  
/* When we mumble things in memory, sometimes we need to create the doc    */
/* manually.                                                               */
/***************************************************************************/  

PROCEDURE CreateDummyXDoc:
                             
  DEFINE INPUT PARAMETER hXMLChunk AS HANDLE NO-UNDO.

  CREATE X-DOCUMENT hXMLDoc.
  CREATE X-NODEREF hRootNode.      
  
  hXMLDoc:IMPORT-NODE (hRootNode, hXMLChunk, TRUE).    
  hXMLDoc:APPEND-CHILD (hRootNode).

END. /* hDOMPhoneNode */            
          
/***************************************************************************/  
/* When XML is sent to a web server, we want a way to set it from the      */
/* WEB-CONTEXT:X-DOCUMENT handle (or other) to the handle these routines   */
/* use.                                                                    */
/***************************************************************************/

PROCEDURE SetInternalXMLHandle:
  
  DEFINE INPUT PARAMETER hXML AS HANDLE NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").
  
  ASSIGN hRootNode = hXML.
  
END. /* PROCEDURE SetInternalXMLHandle */

/***************************************************************************/
/* If one is using these routines to wander up and down branches of the    */
/* document, it is helpful to "remember" the original root node in the     */
/* implementing program and then using SetInternalXMLHandle to the branch  */
/* for use of these routines within ONLY that branch.                      */
/***************************************************************************/   

PROCEDURE GetInternalXMLHandle:
  
  DEFINE OUTPUT PARAMETER hRtn AS HANDLE NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").
  
  ASSIGN hRtn = hRootNode.
  
END. /* PROCEDURE GetInternalXMLHandle */          

/***************************************************************************/
/* This is actually more of a debugging routine to check the resetting of  */
/* the hRoot is happening properly, but implementing programmers might     */
/* find it useful.                                                         */
/* In the future these routines might actually be adapted to writing to    */
/* nodes so this could be useful for that also.                            */
/***************************************************************************/
                                                                 
PROCEDURE SaveXMLDoc:
  
  DEFINE INPUT PARAMETER cFileName AS CHARACTER NO-UNDO.    

  hXMLDoc:SAVE ("file", cFileName).

END. /* SaveXMLDoc */
                                                              
/***************************************************************************/
/* Given a tag name and which occurence, pull back it's value.             */
/***************************************************************************/

PROCEDURE getElementByTagname:

  DEFINE INPUT PARAMETER cTagName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER iOccurence AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER cCDATA AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iCurOccurence AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCurNode      AS INTEGER NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").

  ASSIGN hTrvNode = hRootNode
         iCurOccurence = 0.

  RUN DOMFindTag.p (INPUT-OUTPUT hTrvNode,
                    INPUT cTagName,
                    INPUT-OUTPUT iCurOccurence,
                    INPUT iOccurence,
                    OUTPUT cCData).

END. /* PROCEDURE getElementsbyTagname */

/***************************************************************************/
/* Given a tag, an attribute on the tag, the tag occurence - yank the at-  */
/* tributes value back into the caller.                                    */
/***************************************************************************/

PROCEDURE getAttributeByTagName:

  DEFINE INPUT PARAMETER cTagName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER iOccurence AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER cAttrName AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cCDATA AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iCurOccurence AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCurNode      AS INTEGER NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").

  ASSIGN hTrvNode = hRootNode
         iCurOccurence = 0.

  RUN DOMFindAttr.p (INPUT-OUTPUT hTrvNode,
                     INPUT cTagName,
                     INPUT cAttrName,
                     INPUT-OUTPUT iCurOccurence,
                     INPUT iOccurence,
                     OUTPUT cCData).


END. /* PROCEDURE getAttribute */

/***************************************************************************/
/* Given a tag and it's occurence, return the list of attributes available */
/* on it.  WARNING: Some badly formed XML have the same tag with different */
/* attributes on them!                                                     */
/***************************************************************************/

PROCEDURE ListAttributesOnTag:

  DEFINE INPUT PARAMETER cTagName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER iOccurence AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER cCDATA AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iCurOccurence AS INTEGER NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").

  ASSIGN hTrvNode = hRootNode
         iCurOccurence = 0.

  RUN DOMListAttrs.p (INPUT-OUTPUT hTrvNode,
                      INPUT cTagName,
                      INPUT-OUTPUT iCurOccurence,
                      INPUT iOccurence,
                      OUTPUT cCData).

END. /* PROCEDURE ListAttributesOnTag */

/***************************************************************************/
/* Given a tag name, return the number of times it appears so one can      */
/* easily DO i = 1 TO n... through the tags if need be.                    */
/***************************************************************************/

PROCEDURE NumOfTags:

  DEFINE INPUT PARAMETER cTagName AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER iCountOfTagName AS INTEGER NO-UNDO.

  /*********************************************************************/
  /* This should be recursive but internal procedures tend to be messy */
  /* about that so we iterate as best we can.                          */
  /*********************************************************************/

  ASSIGN hTrvNode = hRootNode.
  ASSIGN iCountOfTagName = 0.

  RUN DOMCountTag.p (INPUT hTrvNode, INPUT cTagName, INPUT-OUTPUT iCountOfTagName).

END. /* PROCEDURE NumOfTags */


/***************************************************************************/
/* There are times when there is a node of the same name within two dif-   */
/* ferent parent nodes.  Since these parent nodes can be switched around,  */
/* sometimes it is not so safe to simply get instance n of the desired node*/
/* because it might depend on which parent node it should be under.        */
/* Once you have this tag handle, one can actually use these routines by   */
/* making that handle the root node as far as these routines are concerned.*/
/* Just be sure to remember the original root node for later processing!   */
/***************************************************************************/

PROCEDURE getHndlByTagName:

  DEFINE INPUT PARAMETER cTagName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER iOccurence AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER hNode AS HANDLE NO-UNDO.

  DEFINE VARIABLE iCurOccurence AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCurNode      AS INTEGER NO-UNDO.

  RUN SetError IN THIS-PROCEDURE (INPUT "000").

  ASSIGN hTrvNode = hRootNode
         iCurOccurence = 0.

  RUN DOMFindTagHndl.p (INPUT-OUTPUT hTrvNode,
                        INPUT cTagName,
                        INPUT-OUTPUT iCurOccurence,
                        INPUT iOccurence,
                        OUTPUT hNode).
                                             
END. /* PROCEDURE getHndlByTagName */

